# Title: Summarize Cubist Models
# Developer: Joshua Picotte
#            Fire Specialist
#            ASRC Federal InuTeq LLC
#            Contractor to the U.S. Geological Survey (USGS)
#            Earth Resources Observation and Science (EROS) Center
# E-mail: joshua.picotte.ctr@usgs.gov
# Version: 1.0
# Release Date: 7/5/2016


# Any use of trade, firm, or product names is for descriptive purposes only
# and does not imply endorsement by the U.S. Government.

# Although this program has been used by the USGS, no warranty,
# expressed or implied, is made by the USGS or the United States Government
# as to the accuracy and functioning of the program and related program
# material nor shall the fact of distribution constitute any such warranty,
# and no responsibility is assumed by the USGS in connection therewith.
import os
import glob
import time
import numpy as np

#* Change me as needed
# Set the Python Program option
print '\n\n***\nOption 1:\nAllows the user to randomly select a' +\
      ' percentage of the data.\nThe only user input is the' +\
      ' number of rules need for the Cubist model.'
print '\n\n***\nOption 2:\nAllows the user to set the percentage of' +\
      ' data used in the random selection.\nIt also allows' +\
      ' the user to set the minimum and maximum number of Cubist' +\
      ' rules\nthat the program will allow.'
Option = int(raw_input('\n*Enter in "1" for Option 1 or "2" for Option 2:\n'))

if Option == 1:
    RulesMax = int(raw_input('\nEnter in the maximum number of model rules.\nIf unsure, enter in 100 (CUBIST Default).\n'))
elif Option == 2:
    # Minimum number of rules
    RulesMin = int(raw_input('\nEnter in the minimum number of model rules.\nIf unsure, enter in 1.\n'))
    # Maximum number of rules
    RulesMax = int(raw_input('\nEnter in the maximum number of model rules.\nIf unsure, enter in 100 (CUBIST Default).\n'))
    # Percent of Dataset used
    # -S option
    Soption = int(raw_input('\nEnter in the percent of data to be used in model developement (e.g. 80).\n'))
    # *
else:
    print '\n***ERROR***\nYou typed in option ' +\
          str(Option) +\
          ', which is not a valid option. Bailing...'
    raise SystemExit

def Option1(InputDir, OutputDir, RulesMax):
    
    # Get subfolders
    Ninput = glob.glob(InputDir + '*')
    
    # Run through subfolders
    for n in Ninput:
        SubDir = n +\
                 '\\'

        SubName = (SubDir.replace(InputDir, '')).strip('\\')
        

        OutSub = SubDir.replace(InputDir, OutDir)

        # Create output subfolder
        if not os.path.exists(OutSub):
            os.mkdir(OutSub)
        else:
            pass

        # Get list of .data files
        # in subfolder
        GetData = glob.glob(SubDir +'*.data')

        for InData in GetData:

            count = 0
            while count < 9:
                count = count + 1

                # -S option
                Sopt = str(10*count)

                print '\nUsing ' +\
                      Sopt +\
                      '% of the data'

                # Need to create a folder for each run
                RunFolder = OutSub +\
                            SubName +\
                            str(Sopt) +\
                            '%' +\
                            '\\'

                # Create subfolder per -S option
                if not os.path.exists(RunFolder):
                    os.mkdir(RunFolder)
                else:
                    pass

                # Run Each -S opt 9 times
                C2 = 0
                while C2 < 9:
                    C2 = C2 + 1

                    # -i Option
                    Iopt = str(C2)

                    print 'Iteration ' +\
                          Iopt 

                    OutFile = RunFolder +\
                              SubName +\
                              '_' +\
                              Sopt +\
                              '%' +\
                              '_' +\
                              Iopt +\
                              '.out'

                    # Cubist command line
                    # runs per -S option
                    # and -i option
                    command = Indir +\
                              'GPL\\cubist\\cubist_gpl.exe -f ' +\
                              InData.replace('.data','') +\
                              ' -S ' +\
                              Sopt +\
                              ' -I ' +\
                              Iopt +\
                              ' -r' +\
                              str(RulesMax) +\
                              ' -e 10 > ' +\
                              OutFile

                    os.system(command)

            # Open Output Textfiles
            # Calculate Mean
            OutCsv = OutSub +\
                     SubName +\
                     '.csv'

            Ocsv = open(OutCsv, 'w')
            Ocsv.write('Sample%,N,MinRule,MaxRule,TrainMean,TestMean,TrainStd,TestStd\n')

            count = 0
            while count < 9:
                count = count + 1

                # -S option
                Sopt = str(10*count)

                # Need to create a folder for each run
                RunFolder = OutSub +\
                            SubName +\
                            str(Sopt) +\
                            '%' +\
                            '\\'

                TrainErr = []
                TestErr = []
                OutRule = []
                # Run Each -S opt 9 times
                C2 = 0
                while C2 < 9:
                    C2 = C2 + 1

                    # -i Option
                    Iopt = str(C2)

                    OutFile = RunFolder +\
                              SubName +\
                              '_' +\
                              Sopt +\
                              '%' +\
                              '_' +\
                              Iopt +\
                              '.out'

                    
                    if os.path.exists(OutFile):
                        # Open OutFile and parse
                        OFile = open(OutFile, 'r')
                        PFile = OFile.readlines()
                        OFile.close()

                        Out = []
                        Rule = []
                        for p in PFile:
                            if '    Average  |error|                ' in p:
                                Out.append(p.replace('    Average  |error|                ','').strip('\n'))
                            elif 'Rule ' in p:
                                RuleN = int((p.split(':')[0].replace('Rule ','')).strip(' '))
                                Rule.append(RuleN)
                            else:
                                pass
                                           

                        # Get evaluation Training
                        TrainErr.append(float(Out[0]))
                        TestErr.append(float(Out[1]))
                        OutRule.append((sorted(Rule, reverse=True))[0])
                        
                    else:
                        pass

                TrainMean = round(np.mean(TrainErr), 3)
                TrainStd = round(np.std(TrainErr), 3)
                TestMean = round(np.mean(TestErr), 3)
                TestStd = round(np.std(TestErr), 3)
                MinRule = np.min(OutRule)
                MaxRule = np.max(OutRule)
                N = len(OutRule)

                Ocsv.write(Sopt +\
                           ',' +\
                           str(N) +\
                           ',' +\
                           str(MinRule) +\
                           ',' +\
                           str(MaxRule) +\
                           ',' +\
                           str(TrainMean) +\
                           ',' +\
                           str(TestMean) +\
                           ',' +\
                           str(TrainStd) +\
                           ',' +\
                           str(TestStd) +\
                           '\n'
                           )

            Ocsv.close()

def Option2(InputDir, OutputDir, RulesMax, RulesMin, Soption):
    # Get subfolders
    Ninput = glob.glob(InputDir + '*')

    # Run through subfolders
    for n in Ninput:
        SubDir = n +\
                 '\\'

        SubName = (SubDir.replace(InputDir, '')).strip('\\')
        

        OutSub = SubDir.replace(InputDir, OutDir) +\
                 'Extract' +\
                 str(Soption) +\
                 '\\'

        print '\nUsing ' +\
              str(Soption) +\
              '% of the data'

        # Create output subfolder
        if not os.path.exists(OutSub):
            os.mkdir(OutSub)
        else:
            pass

        # Get list of .data files
        # in subfolder
        GetData = glob.glob(SubDir +'*.data')

        for InData in GetData:

            # Open Output Textfiles
            # Calculate Mean
            OutCsv = SubDir.replace(InputDir, OutDir)+\
                     SubName +\
                     '_' +\
                     str(Soption) +\
                     '%.csv'

            Ocsv = open(OutCsv, 'w')
            Ocsv.write('Sample%,RuleNum,TrainErr,TestErr\n')

            C2 = RulesMin - 1
            while C2 >= RulesMin - 1 and C2 < RulesMax - 1 :
                C2 = C2 + 1

                # Rules
                Ropt = str(C2)
                
                OutFile = OutSub +\
                          SubName +\
                          '_' +\
                          str(Soption) +\
                          '%_' +\
                          Ropt +\
                          '_rules.out'

                print 'Currently using ' +\
                      Ropt +\
                      ' rules...' 

                # Cubist command line
                # runs for a set percentage (e.g. 60) of the data
                # Use -S option
                # and sets the number of rules (e.g. between 1 and 12) -r option

                command = Indir +\
                          'GPL\\cubist\\cubist_gpl -f ' +\
                          InData.replace('.data','') +\
                          ' -S ' +\
                          str(Soption) +\
                          ' -I 1 -r ' +\
                          Ropt +\
                          ' -e 10 > ' +\
                          OutFile

                os.system(command)

                if os.path.exists(OutFile):
                    # Open OutFile and parse
                    OFile = open(OutFile, 'r')
                    PFile = OFile.readlines()
                    OFile.close()

                    Out = []
                    Rule = []
                    for p in PFile:
                        if '    Average  |error|                ' in p:
                            Out.append(p.replace('    Average  |error|                ','').strip('\n'))
                        else:
                            pass

                    Train = round(float(Out[0]), 3)
                    Test = round(float(Out[1]), 3)

                    Ocsv.write(str(Soption) +\
                               ',' +\
                               str(Ropt) +\
                               ',' +\
                               str(Train) +\
                               ',' +\
                               str(Test) +\
                               '\n'
                               )
                    
                else:
                    pass
            
            Ocsv.close()

# Start Timer
t0 = time.time()

Indir = os.getcwd() +\
        '\\'

InputDir = Indir +\
           'Input\\'

OutDir = InputDir.replace('Input', 'Output')

# Make sure Input and Output Directories exist
if not os.path.exists(InputDir):
    os.mkdir(InputDir)
else:
    pass

if not os.path.exists(OutDir):
    os.mkdir(OutDir)
else:
    pass

# Run the program
# based on which option the user chooses...
if Option == 1:
    Option1(InputDir, OutDir, RulesMax)
elif Option == 2:
    Option2(InputDir, OutDir, RulesMax, RulesMin, Soption)
else:
    print '\n***ERROR***\nYou typed in option ' +\
          str(Option) +\
          ', which is not a valid option. Bailing...'
    raise SystemExit


t1 = time.time()

OutTime = t1 - t0
print "\nTotal Process Time: " + str(OutTime) + " seconds..."              
